Package: dittoSeq
Type: Package
Title: User Friendly Single-Cell and Bulk RNA Sequencing Visualization
Version: 1.23.0
Authors@R: c(person("Daniel", "Bunis", email = "daniel.bunis@ucsf.edu", role=c("aut", "cre")),
    person("Jared", "Andrews", email = "jared.andrews07@gmail.com", role=c("aut", "ctb")))
Description: A universal, user friendly, single-cell and bulk RNA sequencing
    visualization toolkit that allows highly customizable creation of color
    blindness friendly, publication-quality figures. dittoSeq accepts both
    SingleCellExperiment (SCE) and Seurat objects, as well as the import and
    usage, via conversion to an SCE, of SummarizedExperiment or DGEList bulk
    data. Visualizations include dimensionality reduction plots, heatmaps,
    scatterplots, percent composition or expression across groups, and more.
    Customizations range from size and title adjustments to automatic
    generation of annotations for heatmaps, overlay of trajectory analysis
    onto any dimensionality reduciton plot, hidden data overlay upon cursor
    hovering via ggplotly conversion, and many more. All with simple, discrete
    inputs. Color blindness friendliness is powered by legend adjustments
    (enlarged keys), and by allowing the use of shapes or letter-overlay in
    addition to the carefully selected dittoColors().
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: ggplot2
Imports: methods, colorspace (>= 1.4), gridExtra, cowplot, reshape2,
        pheatmap, grDevices, ggrepel, ggridges, stats, utils,
        SummarizedExperiment, SingleCellExperiment, S4Vectors
Suggests: plotly, testthat, Seurat (>= 2.2), DESeq2, edgeR,
        ggplot.multistats, knitr, rmarkdown, BiocStyle, scRNAseq,
        ggrastr (>= 0.2.0), ComplexHeatmap, bluster, scater, scran,
        MASS
biocViews: Software, Visualization, RNASeq, SingleCell, GeneExpression,
        Transcriptomics, DataImport
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/dittoSeq
git_branch: devel
git_last_commit: 9ee30ee
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: no
Packaged: 2025-11-03 22:17:06 UTC; biocbuild
Author: Daniel Bunis [aut, cre],
  Jared Andrews [aut, ctb]
Maintainer: Daniel Bunis <daniel.bunis@ucsf.edu>
