# Generated by roxygen2: do not edit by hand

export(Darken)
export(Lighten)
export(Simulate)
export(addDimReduction)
export(addPrcomp)
export(demux.SNP.summary)
export(demux.calls.summary)
export(dittoBarPlot)
export(dittoBoxPlot)
export(dittoColors)
export(dittoDimHex)
export(dittoDimPlot)
export(dittoDotPlot)
export(dittoFreqPlot)
export(dittoHeatmap)
export(dittoPlot)
export(dittoPlotVarsAcrossGroups)
export(dittoRidgeJitter)
export(dittoRidgePlot)
export(dittoScatterHex)
export(dittoScatterPlot)
export(gene)
export(getGenes)
export(getMetas)
export(getReductions)
export(importDemux)
export(importDittoBulk)
export(isBulk)
export(isGene)
export(isMeta)
export(meta)
export(metaLevels)
export(multi_dittoDimPlot)
export(multi_dittoDimPlotVaryCells)
export(multi_dittoPlot)
export(setBulk)
import(ggplot2)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(S4Vectors,DataFrame)
importFrom(SingleCellExperiment,"int_metadata<-")
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,int_metadata)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(colorspace,deutan)
importFrom(colorspace,protan)
importFrom(colorspace,tritan)
importFrom(cowplot,get_legend)
importFrom(cowplot,ggdraw)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(ggridges,geom_density_ridges2)
importFrom(grDevices,colorRampPalette)
importFrom(gridExtra,grid.arrange)
importFrom(methods,as)
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,sd)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
